@@ -23,6 +23,7 @@ from keystoneauth1 import exceptions
 from oslo_log import log as logging
 from oslo_serialization import jsonutils
 import six
+import yaml
 
 from congressclient.common import utils
 
@@ -79,6 +80,9 @@ class CreatePolicyRule(show.ShowOne):
 
     def take_action(self, parsed_args):
         self.log.debug('take_action(%s)' % parsed_args)
+        # set default max-width
+        if parsed_args.max_width == 0:
+            parsed_args.max_width = 80
         client = self.app.client_manager.congressclient
         body = {'rule': parsed_args.rule}
         if parsed_args.rule_name:
@@ -223,6 +227,9 @@ class ListPolicyTables(lister.Lister):
 
     def take_action(self, parsed_args):
         self.log.debug('take_action(%s)' % parsed_args)
+        # set default max-width
+        if parsed_args.max_width == 0:
+            parsed_args.max_width = 80
         client = self.app.client_manager.congressclient
         data = client.list_policy_tables(parsed_args.policy_name)['results']
         columns = ['id']
@@ -243,6 +250,9 @@ class ListPolicy(lister.Lister):
         return parser
 
     def take_action(self, parsed_args):
+        # set default max-width
+        if parsed_args.max_width == 0:
+            parsed_args.max_width = 80
         client = self.app.client_manager.congressclient
         data = client.list_policy()['results']
         columns = ['id', 'name', 'owner_id', 'kind', 'description']
@@ -277,13 +287,17 @@ class CreatePolicy(show.ShowOne):
         parser.add_argument(
             '--kind',
             metavar="<kind>",
-            choices=['nonrecursive', 'database', 'action', 'materialized'],
+            choices=['nonrecursive', 'database', 'action', 'materialized',
+                     'z3'],
             help="Kind of policy: "
-                 "{nonrecursive, database, action, materialized}")
+                 "{nonrecursive, database, action, materialized, z3}")
         return parser
 
     def take_action(self, parsed_args):
         self.log.debug('take_action(%s)' % parsed_args)
+        # set default max-width
+        if parsed_args.max_width == 0:
+            parsed_args.max_width = 80
         client = self.app.client_manager.congressclient
         body = {'name': parsed_args.policy_name,
                 'description': parsed_args.description,
@@ -293,6 +307,47 @@ class CreatePolicy(show.ShowOne):
         return zip(*sorted(six.iteritems(data)))
 
 
+class CreatePolicyFromFile(show.ShowOne):
+    """Create a policy."""
+
+    log = logging.getLogger(__name__ + '.CreatePolicy')
+
+    def get_parser(self, prog_name):
+        parser = super(CreatePolicyFromFile, self).get_parser(prog_name)
+        parser.add_argument(
+            'policy_file_path',
+            metavar="<policy_file_path>",
+            help="Path to policy file")
+        return parser
+
+    def take_action(self, parsed_args):
+        self.log.debug('take_action(%s)' % parsed_args)
+        # set default max-width
+        if parsed_args.max_width == 0:
+            parsed_args.max_width = 80
+        client = self.app.client_manager.congressclient
+        with open(parsed_args.policy_file_path, "r") as stream:
+            policies = yaml.load_all(stream)
+            try:
+                body = next(policies)
+            except StopIteration:
+                raise Exception('No policy found in file.')
+            try:
+                body = next(policies)
+                raise Exception(
+                    'More than one policy found in file. None imported.')
+            except StopIteration:
+                pass
+        data = client.create_policy(body)
+
+        def rule_dict_to_string(rules):
+            rule_str_list = [rule['rule'] for rule in rules]
+            return "\n".join(rule_str_list)
+
+        data['rules'] = rule_dict_to_string(data['rules'])
+        return zip(*sorted(six.iteritems(data)))
+
+
 class DeletePolicy(command.Command):
     """Delete a policy."""
 
@@ -337,6 +392,9 @@ class ListPolicyRows(lister.Lister):
 
     def take_action(self, parsed_args):
         self.log.debug('take_action(%s)' % parsed_args)
+        # set default max-width
+        if parsed_args.max_width == 0:
+            parsed_args.max_width = 80
         client = self.app.client_manager.congressclient
         answer = client.list_policy_rows(parsed_args.policy_name,
                                          parsed_args.table,
@@ -373,6 +431,9 @@ class ShowPolicyRule(show.ShowOne):
 
     def take_action(self, parsed_args):
         self.log.debug('take_action(%s)' % parsed_args)
+        # set default max-width
+        if parsed_args.max_width == 0:
+            parsed_args.max_width = 80
         client = self.app.client_manager.congressclient
         results = client.list_policy_rules(parsed_args.policy_name)
         rule_id = utils.get_resource_id_from_name(
@@ -401,6 +462,9 @@ class ShowPolicyTable(show.ShowOne):
 
     def take_action(self, parsed_args):
         self.log.debug('take_action(%s)' % parsed_args)
+        # set default max-width
+        if parsed_args.max_width == 0:
+            parsed_args.max_width = 80
         client = self.app.client_manager.congressclient
         data = client.show_policy_table(parsed_args.policy_name,
                                         parsed_args.table_id)
@@ -423,6 +487,9 @@ class ShowPolicy(show.ShowOne):
 
     def take_action(self, parsed_args):
         self.log.debug('take_action(%s)' % parsed_args)
+        # set default max-width
+        if parsed_args.max_width == 0:
+            parsed_args.max_width = 80
         client = self.app.client_manager.congressclient
         results = client.list_policy()
         policy_id = utils.get_resource_id_from_name(
